@@ -1382,11 +1382,6 @@ QDDSHandler::QDDSHandler() :
 {
 }
 
-QByteArray QDDSHandler::name() const
-{
-    return QByteArrayLiteral("dds");
-}
-
 bool QDDSHandler::canRead() const
 {
     if (m_scanState == ScanNotScanned && !canRead(device()))
